@@ -9,8 +9,11 @@ import subprocess
 import shlex
 import shutil
 import sys
+from typing import cast, Optional, Dict, List, Set
 
 from ..common import (
+    Package,
+    PyVerTuple,
     find_packages,
     compute_python_package_path,
     compute_ament_cmake_python_path,
@@ -23,12 +26,12 @@ from ..common import (
 
 # command line identation
 cmdind = "    "
-destdirs = set()
-catkin_srcs = []
+destdirs: Set[Path] = set()
+catkin_srcs: List[str] = []
 dpkg_info = query_dpkg_architecture()
 
 
-def prepare_argparse():
+def prepare_argparse() -> argparse.ArgumentParser:
     p = argparse.ArgumentParser()
     p.add_argument(
         "--dry-run", action="store_true", help="do not execute any commands for real"
@@ -80,13 +83,13 @@ def prepare_argparse():
     return p
 
 
-def main():
+def main() -> int:
     p = prepare_argparse()
     args = p.parse_args()
-    return run(args, os.environ)
+    return run(args, cast(Dict[str, str], os.environ))
 
 
-def run(args, environ):
+def run(args: argparse.Namespace, environ: Dict[str, str]) -> int:
     # initialize script
     logging.basicConfig(
         format="%(levelname).1s: dh_ros %(module)s:%(lineno)d: %(message)s"
@@ -106,11 +109,13 @@ def run(args, environ):
 
     if args.build_types:
         sys.stdout.write(
-            "\n".join(sorted(set(p.get_build_type() for _, p in packages)))
+            "\n".join(sorted(set(cast(str, p.get_build_type()) for _, p in packages)))
         )
         return 0
 
-    buildopts = re.split(r"[ ,\t\r\n]+", environ.get("DEB_BUILD_OPTIONS", ""))
+    buildopts: List[str] = re.split(
+        r"[ ,\t\r\n]+", environ.get("DEB_BUILD_OPTIONS", "")
+    )
 
     packages = sort_packages(packages, test_depends="nocheck" not in buildopts)
     skipped_packages = get_build_var(None, "ROS_SKIP_PACKAGES", env=environ)
@@ -137,7 +142,7 @@ def run(args, environ):
             shlex.split(ignore_test_results) if ignore_test_results else []
         )
     install_prefix = Path(
-        get_build_var(None, "ROS_INSTALL_PREFIX", args.install_prefix, env=environ)
+        get_build_var(None, "ROS_INSTALL_PREFIX", str(args.install_prefix), env=environ)
     ).resolve()
 
     if args.build_order:
@@ -197,7 +202,7 @@ def run(args, environ):
             if package.get_build_type() == "catkin":
                 catkin_srcs.append(str(path.resolve()))
             python_versions = [None]
-            package_buildopts = []
+            package_buildopts: List[str] = []
             if package.name in skipped_tests:
                 package_buildopts.append("nocheck")
             if (
@@ -210,14 +215,15 @@ def run(args, environ):
                 ]
             ):
                 default_python, supported_python = get_python_versions()
-                python_versions = [
-                    tuple(v) for v in supported_python if v != default_python
-                ]
-                python_versions.append(tuple(default_python))
-                s = [".".join(str(v) for v in pv) for pv in python_versions]
-                log.info(
-                    f"Python extensions will be built against Python {', '.join(s)}"
-                )
+                if default_python is not None and supported_python is not None:
+                    python_versions = [
+                        tuple(v) for v in supported_python if v != default_python
+                    ]
+                    python_versions.append(tuple(default_python))
+                    s = [".".join(str(v) for v in pv) for pv in python_versions]
+                    log.info(
+                        f"Python extensions will be built against Python {', '.join(s)}"
+                    )
 
             for python_version in python_versions:
                 do_action(
@@ -286,16 +292,25 @@ def run(args, environ):
 
 
 def do_action(
-    action, args, path, package, buildopts, install_prefix, environ, python_version
-):
+    action: str,
+    args: argparse.Namespace,
+    path: Path,
+    package: Package,
+    buildopts: List[str],
+    install_prefix: Path,
+    environ: Dict[str, str],
+    python_version: Optional[PyVerTuple],
+) -> None:
     global destdirs
     global catkin_srcs
     log = logging.getLogger("dhros")
     destdir = Path(
-        get_build_var(package.name, "ROS_DESTDIR", args.destdir, env=environ)
+        get_build_var(package.name, "ROS_DESTDIR", str(args.destdir), env=environ)
     ).resolve()
     homedir = Path(
-        get_build_var(package.name, "ROS_HOME", args.builddir / ".roshome", env=environ)
+        get_build_var(
+            package.name, "ROS_HOME", str(args.builddir / ".roshome"), env=environ
+        )
     ).resolve()
     builddir = args.builddir / package.name
     python_interpreter = ""
@@ -311,7 +326,7 @@ def do_action(
         get_build_var(
             package.name,
             "ROS_CATKIN_TEST_RESULTS_DIR",
-            builddir / "test_results",
+            str(builddir / "test_results"),
             env=environ,
         )
     ).resolve()
@@ -319,7 +334,7 @@ def do_action(
     relative_install_prefix = install_prefix.relative_to("/")
     destdirs.add(destdir)
     package_suffix = re.sub(r"[^A-Za-z0-9]", "_", package.name)
-    build_type = package.get_build_type()
+    build_type = cast(str, package.get_build_type())
     hook_vars = {
         "build_dir": shlex.quote(str(builddir.resolve())),
         "build_type": shlex.quote(build_type),
@@ -552,7 +567,13 @@ def do_action(
                 shutil.rmtree(pycache, ignore_errors=True)
 
 
-def execute_hook(hook, package, vars, env, dry_run=False):
+def execute_hook(
+    hook: str,
+    package: Package,
+    vars: Dict[str, str],
+    env: Dict[str, str],
+    dry_run: bool = False,
+):
     log = logging.getLogger("dhros")
     cmdline = get_build_var(package.name, f"ROS_EXECUTE_{hook.upper()}", env=env)
     if cmdline is None:
